@@ -359,7 +359,8 @@ void search_filename(bool forwards)
 #endif
 	}
 
-	findfile(last_search, forwards);
+	if (response == 0 || response == -2)
+		findfile(last_search, forwards);
 }
 
 /* Search again without prompting for the last given search string,
@@ -380,6 +381,18 @@ void research_filename(bool forwards)
 	}
 }
 
+/* Select the first file in the list -- called by ^W^Y. */
+void to_first_file(void)
+{
+	selected = 0;
+}
+
+/* Select the last file in the list -- called by ^W^V. */
+void to_last_file(void)
+{
+	selected = list_length - 1;
+}
+
 /* Strip one element from the end of path, and return the stripped path.
  * The returned string is dynamically allocated, and should be freed. */
 char *strip_last_component(const char *path)
